#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "MatVec.h"
#include "optimize.h"
#include "gauss.c"
#include <Eigen>


using Eigen::MatrixXd;
using namespace Eigen;
using namespace std;

#define MAXFILELEN 2001 //maximum length of file names
#define MAXINDLEN 20 //maximum length of individual ID
#define MAXGENOLEN 30  //maximum length of genotype of a single SNP
#define MAXMARKERLEN 30 //maximum length of marker ID
#define MAXPHELEN 1000 //maximum length of lines in phenotype file
#define POSDEF_CUTOFF 1e-8

extern "C" {
void dsyevr_( char* jobz, char* range, char* uplo, int* n, double* a, int* lda, double* vl, double* vu, int* il, int* iu, double* abstol, int* M,
                double* w, double* z, int* ldz, int* isuppz, double* work, int* lwork, int* iwork, int* liwork, int* info );
}

typedef Map<Matrix<double, Dynamic, Dynamic, RowMajor> > MapRowXd;
typedef Map<Matrix<double, Dynamic, Dynamic, ColMajor> > MapColXd;
typedef Map<const Matrix<double, Dynamic, Dynamic, RowMajor> > MapRowXdConst;

int savemem=0;
void count_cov(char pheno_filename[], int *n_cov);
void readpheno(char pheno_filename[], int n_fam, int n_cov, struct FAMILY *family, int *n_indiv_total);
void readkin(char kin_filename[], int n_fam, struct FAMILY *family);
int findInd(int n_indiv, char **indivlist, char indiv[]);
void eig(int n_fam, struct FAMILY *family, char eig_filename[]);
void eigcomp_readgeno(int n_fam, int n_indiv_total, struct FAMILY *family, char eiggeno_filename[]);
void eigcomp_writeGRM(int n_fam, struct FAMILY *family, char *outgrm_filename);
void leigcomp_kin(int n_fam, struct FAMILY *family, char outeig_filename[]);
void eigcomp_kin(int n_fam, struct FAMILY *family, char outeig_filename[]);
int estimate(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param, char param_filename[]);
int estimate_Beta(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param);
void estimate_sigmaT2(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param);
void estimate_xi (int n_fam, struct FAMILY *family, struct PARAM *param);
double xi_score(double xi, void *info);
void optimize_xi (int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param, int *beta_converge);
double xi_score_optim(double xi, void *info);
void PrepareTest(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param, double *varT);
int Test(int n_fam, int n_cov, int n_indiv_total, char geno_filename[], char p_filename[], 
	struct FAMILY *family, struct PARAM *param, double *varT);
void Logistic (int n_fam, int n_cov, int n_indiv_total, struct FAMILY *family, struct PARAM *param);




struct FAMILY {
	int n_indiv;
	char **indiv;
	double **kin, **eigvector, *eigvalue;
	double **cov, *pheno, *D_inv, *Z; //Z = Ut * M_inv * (y-mu), where Phi = U * L *Ut
	double *M, *y4;
};

struct PARAM{
	double *beta, *sigmaT2, *xi, *beta_initial;
};

struct INFO{
	int n_fam;
	int n_cov;
	struct FAMILY *family;
	struct PARAM *param;
	int *beta_converge;
};
	

int main(int argc, char **argv){
    printf("\n"
           "--------------------------------------------------------------------------------\n"
           "|                                   CARAT                                      |\n"
		   "|                 (CAse-control Retrospective Association Test)                |\n"
           "|                       Version 1.3 - May 20, 2016                             |\n"
           "|                                                                              |\n"
           "|                            Copyright(C) 2016                                 |\n"
           "|              Duo Jiang, Sheng Zhong and Mary Sara McPeek                     |\n"
           "|                                                                              |\n"
           "|                                 Homepage:                                    |\n"
           "|              http://galton.uchicago.edu/~mcpeek/software/CARAT               |\n"
           "--------------------------------------------------------------------------------\n\n"
           );	
	char pheno_filename[MAXFILELEN]="pheno.txt", geno_filename[MAXFILELEN]="geno.txt", 
		eig_filename[MAXFILELEN], kin_filename[MAXFILELEN], eiggeno_filename[MAXFILELEN];
	char p_filename[MAXFILELEN], param_filename[MAXFILELEN], out_prefix[MAXFILELEN], outeig_filename[MAXFILELEN], outgrm_filename[MAXFILELEN];
	int arg, pfile=0, gfile=0, erecyc=0, efile=0, kfile=0, egfile=0, ogrmfile=0, ofile=0, lapa=1, null_only=0, meow=0, n_fam = 1, n_cov, i, j, n_indiv_total, initial_error=0;
	struct FAMILY *family, *family_tmp;
	struct PARAM *param;
	double varT;
	
	if (argc > 1) {
		for (arg = 1; arg < argc && argv[arg][0] == '-'; arg++) {
			switch (argv[arg][1]) {
			case 'p':
				pheno_filename[MAXFILELEN-1]!='\0';
				strncpy(pheno_filename, argv[++arg], MAXFILELEN);
				if(pheno_filename[MAXFILELEN-1]!='\0'){
					printf("ERROR: Input file names cannot exceed %d characters. Please double check the phenotype and covariate data file!\n", MAXFILELEN-1);
					exit(1);
				}
				pfile = 1;
				break;
			case 'g':
				geno_filename[MAXFILELEN-1]='\0';
				strncpy(geno_filename, argv[++arg], MAXFILELEN);
				if(geno_filename[MAXFILELEN-1]!='\0'){
					printf("ERROR: Input file names cannot exceed %d characters. Please double check the genotype data file!\n", MAXFILELEN-1);
					exit(1);
				}				
				gfile = 1;
				break;										
			case 'e':
			eig_filename[MAXFILELEN-1]='\0';
				strncpy(eig_filename, argv[++arg], MAXFILELEN);
				if(eig_filename[MAXFILELEN-1]!='\0'){
					printf("ERROR: Input file names cannot exceed %d characters. Please double check the eigen-decomposition file!\n", MAXFILELEN-1);
					exit(1);
				}				
				erecyc = 1;
				efile = 1;
				break;
			case 'E':
				lapa = 0;
				break;
			case 'R':
				kin_filename[MAXFILELEN-1]='\0';
				strncpy(kin_filename, argv[++arg], MAXFILELEN);
				if(kin_filename[MAXFILELEN-1]!='\0'){
					printf("ERROR: Input file names cannot exceed %d characters. Please double check the GRM file!\n", MAXFILELEN-1);
					exit(1);
				}				
				kfile = 1;
				break;		
			case 'G': 
				eiggeno_filename[MAXFILELEN-1]='\0';
				strncpy(eiggeno_filename, argv[++arg], MAXFILELEN);
				if(eiggeno_filename[MAXFILELEN-1]!='\0'){
					printf("ERROR: Input file names cannot exceed %d characters. Please double check the GRM genotype file!\n", MAXFILELEN-1);
					exit(1);
				}				
				egfile = 1;
				break;
			case 'r':
				ogrmfile = 1;
				break;	
			case 'o':
				strncpy(out_prefix, argv[++arg], MAXFILELEN);
				ofile = 1;
				if (strlen(out_prefix)>MAXFILELEN-10) {
					printf("ERROR: The prefix for the output files cannot be longer than %d characters.\n", MAXFILELEN-10);
					exit(1);
				}
				break;
			case 'n':
				null_only = 1;
				break;
			case 'M':
				meow = 1;
				break;
			default:
				printf("WARNING: Unknown option \"%s\" ignored!\n", argv[arg]);
				break;
			}
		}
	}
	if(meow==1) printf("         /\\_/\\ " "               /\\_/\\ \n" "    ____/ o o \\ " "         ____/ o o \\ \n" 
	"  /~____  =?= / " "       /~____  =?= / \n" " (______)__m_m) " "      (______)__m_m) \n");
	
	if (!pfile) {
		printf("Default phenotype and covariate data file: %s\n", pheno_filename);
	} else{
		printf("User specified phenotype and covariate data file: %s\n", pheno_filename);
	}
	
	if(null_only==1){
		printf("The program will only fit the null model, and no association tests will be performed. ");
		if(gfile){
			printf("The genotype data file is ignored.\n\n");
		}else{
			printf("\n\n");
		}
	} else{
		if (!gfile) {
			printf("Default genotype data file: %s\n\n", geno_filename);
		} else{
			printf("User specified genotype data file: %s\n\n", geno_filename);
		}
	}
	

	if(ogrmfile&&!egfile){
		printf("WARNING: Option -r ignored because -G is not used.\n");
	}		
	if(efile){
		if(egfile||kfile){
			printf("WARNING: Only ONE of the options, -e, -G and -R, can be used. Program has ignored -G and/or -R. Option -e will be taken. \n");
		}
		printf("Program will use eigen-decomposition results provided in the file: %s\n", eig_filename);
		printf("User should make sure this file has the required format.\n\n");
	} else if(kfile){
		if(egfile){
			printf("WARNING: Only ONE of the options, -e, -G and -R, can be used. Program has ignored -G. Option -G will be taken.\n");
		}
		printf("Eigen-decomposition will be performed based on the genetic relationship matrix (GRM) provided in the file: %s\n", kin_filename);
		printf("User should make sure this file has the required format.\n\n");
	} else if(egfile){
			printf("Eigen-decomposition will be performed on a genetic relationship matrix (GRM) constructed from the genotype data provided in the file: %s\n", eiggeno_filename);
			printf("User should make sure this file has the required format.\n\n");
	} else {
		printf("ERROR: If an eigen-decomposition file is not provided by -e, the user needs to provide either the genetic relationship matrix (flag -r), or genotype data to compute the genetic relationship matrix (flag -G).\n\n");
		exit(1);
	}

	if(!ofile){
		strcpy(p_filename, "CARAT_out.txt");
		strcpy(param_filename, "CARAT_param.txt");
		strcpy(outeig_filename, "CARAT_eig");
		strcpy(outgrm_filename, "CARAT_GRM.txt");
	}else{
		strcpy(p_filename, out_prefix);
		strcat(p_filename, "_out.txt");
		strcpy(param_filename, out_prefix);
		strcat(param_filename, "_param.txt");
		strcpy(outeig_filename, out_prefix);
		strcat(outeig_filename, "_eig");
		strcpy(outgrm_filename, out_prefix);
		strcat(outgrm_filename, "_GRM.txt");
	}
	
	printf("Loading phenotype and covariate data...\n");
	count_cov(pheno_filename, &n_cov);
	family = (struct FAMILY*) malloc(n_fam * sizeof(struct FAMILY));
	readpheno(pheno_filename, n_fam, n_cov, family, &n_indiv_total);
	printf("1 phenotype and %d covariates (intercept not included) found for %d individuals.\n\n", n_cov, n_indiv_total);


	if(efile){
		printf("Loading eigen-decomposition results...\n");
		eig(n_fam, family, eig_filename);
		printf("Completed.\n\n");
	} else if(kfile){
		printf("Loading GRM...\n");
		readkin(kin_filename, n_fam, family);
		printf("GRM loaded.\n");
		printf("Performing eigen-decomposition...\n");
		if(lapa){
			leigcomp_kin(n_fam, family, outeig_filename);
		} else{
			eigcomp_kin(n_fam, family, outeig_filename);
		}
		for(i=0; i<n_fam; i++){
			free_d2array((family+i)->kin, (family+i)->n_indiv, (family+i)->n_indiv);
		}
		printf("Eigen-decomposition completed. Results stored in %s\n\n", outeig_filename);
	} else if(egfile){
		printf("Constructing GRM...\n");
		eigcomp_readgeno(n_fam, n_indiv_total, family, eiggeno_filename);
		if(ogrmfile){
			eigcomp_writeGRM(n_fam, family, outgrm_filename);
			printf("GRM contructed. Results stored in %s\n", outgrm_filename);
		} 
		printf("Performing eigen-decomposition...\n");
		if(lapa){
			leigcomp_kin(n_fam, family, outeig_filename);
		} else{
			eigcomp_kin(n_fam, family, outeig_filename);
		}		for(i=0; i<n_fam; i++){
			free_d2array((family+i)->kin, (family+i)->n_indiv, (family+i)->n_indiv);
		}
		printf("Eigen-decomposition completed. Results stored in %s\n\n", outeig_filename);
	} 
	// Check non-negativeness of eigenvalues
	for(i=0; i<n_fam; i++){
		family_tmp = family+i;
		for (j=0; j<family_tmp->n_indiv; j++){
			if((family_tmp->eigvalue)[j]< -1e-4){
				printf("WARNING: Genetic relationship matrix is not symmetric positive semi-definite!\n");
			}
		}
	}
			
	// initializing parameter estimates with logistic regression
	printf("Estimating parameters under the null...\n");
	param = (struct PARAM*)malloc(sizeof(struct PARAM));
	param -> beta_initial = d1array(n_cov+1); //beta initialized to be zero
	param -> xi = d1array(1);
	param -> sigmaT2 = d1array(1);		
	Logistic (n_fam, n_cov, n_indiv_total, family, param);	
	// Allocate memory for objects to be used in estimation and tests
	for(i=0; i<n_fam; i++){
		family_tmp = family+i;
		family_tmp -> M = d1array(family_tmp->n_indiv);
		family_tmp -> y4 = d1array(family_tmp->n_indiv);
		family_tmp -> D_inv = d1array(family_tmp->n_indiv);
		family_tmp -> Z = d1array(family_tmp->n_indiv);		
	}	
	//Parameter estimation begins		
	param -> beta = d1array(n_cov+1);
	estimate(n_fam, n_cov, family, param, param_filename);
	printf("Parameter estimation completed. Results stored in %s\n\n", param_filename);
	
	if(!null_only){
		// Begin association testing
		printf("Performing association tests...\n");
		PrepareTest(n_fam, n_cov, family, param, &varT);
		// read genotypes. conduct test
		Test(n_fam, n_cov, n_indiv_total, geno_filename, p_filename, family, param, &varT);
	}

	for (i=0; i<n_fam; i++){
		family_tmp = family+i;
		free(family_tmp->pheno);
		free_d2array(family_tmp->cov, family_tmp->n_indiv, n_cov+1);
		for (j=0; j<family_tmp->n_indiv; j++) free((family_tmp->indiv)[j]);
		free(family_tmp->indiv);
		free_d2array(family_tmp->eigvector, family_tmp->n_indiv, family_tmp->n_indiv);
		free(family_tmp->eigvalue);
		free(family_tmp -> D_inv);
		free(family_tmp -> Z);
		free(family_tmp->M);
		free(family_tmp->y4);
	}
	free(family);
	free(param->beta);
	free(param->xi);
	free(param->sigmaT2);
	free(param);
	free(param->beta_initial);
	



}

// Count number of covariates
void count_cov(char pheno_filename[], int *n_cov){
	FILE *phenofile;
	int length, fam, n, sex;
	double pheno, cov;
	char *line, indiv[MAXINDLEN], mom[MAXINDLEN], dad[MAXINDLEN];
	*n_cov = 0;
	length = MAXPHELEN;
	line = (char*)malloc(length*sizeof(char));	
	if( (phenofile = fopen(pheno_filename, "r"))==NULL) {
		printf("ERROR: Can't open phenotype data file!\n");
		exit(1);
	}	
	fgets(line, length, phenofile);		
	n = sscanf(line,"%d %s %s %s %d %[^\n]", &fam, indiv, mom, dad, &sex, line);
	n = sscanf(line, "%lf %[^\n]", &pheno, line);
	if (n<1) {	
		printf("WARNING: No covariate in phenotype data file!\n");
	} else {
		while (n==2){
			n=sscanf(line,"%lf %[^\n]", &cov, line);
			if (n>=1) (*n_cov)++;
		}
	}	
	fclose(phenofile);
	
}

// Read phenotype and covariate data
void readpheno(char pheno_filename[], int n_fam, int n_cov, struct FAMILY *family, int *n_indiv_total){
	FILE *phenofile;
	int i, fam_tmp, j, *indiv_index, sex, k, cov_check;
	double pheno_tmp, cov_tmp;
	char indiv_tmp[MAXINDLEN], mom[MAXINDLEN], dad[MAXINDLEN];
	struct FAMILY *family_tmp;
	if( (phenofile = fopen(pheno_filename, "r"))==NULL) {
		printf("ERROR: Can't open phenotype data file!\n");
		exit(1);
	}
	for (i=0; i<n_fam; i++) (family+i)->n_indiv = 0;	
	(*n_indiv_total) = 0;
	while (!feof(phenofile)){
		if(fscanf(phenofile, "%d %s %s %s %d %lf", &fam_tmp, indiv_tmp, mom, dad, &sex, &pheno_tmp)!=6) break;
		(family+fam_tmp-1)->n_indiv ++;
		(*n_indiv_total)++; 
		for (i=0; i<n_cov; i++){
			fscanf(phenofile, "%lf", &cov_tmp);
		}
	}
	fclose(phenofile);
	
	for (i=0; i<n_fam; i++){
		family_tmp = family + i;
		family_tmp -> pheno = d1array(family_tmp->n_indiv);
		family_tmp -> cov = d2array(family_tmp->n_indiv, n_cov+1);		
		family_tmp->indiv = (char**)malloc(family_tmp->n_indiv * sizeof(char*));
		for (j=0; j<family_tmp->n_indiv; j++){
			(family_tmp->indiv)[j] = (char *)malloc(MAXINDLEN * sizeof(char));
		}
	}
	indiv_index = i1array(n_fam);
	phenofile = fopen(pheno_filename, "r");
	while (!feof(phenofile)){
		if(fscanf(phenofile, "%d %s %s %s %d %lf", &fam_tmp, indiv_tmp, mom, dad, &sex, &pheno_tmp)!=6) break;
		if(pheno_tmp!=0 && pheno_tmp!=1){
			printf("ERROR: Phenotype for individual %s is NOT coded in 0/1 (no missing phenotype is allowed by the program.)!\n", indiv_tmp);
			exit(1);
		}	
		indiv_index[fam_tmp-1]++;
		family_tmp = family+fam_tmp-1;
		strcpy((family_tmp->indiv)[indiv_index[fam_tmp-1]-1], indiv_tmp);
		(family_tmp->pheno)[indiv_index[fam_tmp-1]-1] = pheno_tmp;
		(family_tmp->cov)[indiv_index[fam_tmp-1]-1][0] = 1;
		for (i=0; i<n_cov; i++){
			fscanf(phenofile, "%lf", &cov_tmp);
			(family_tmp->cov)[indiv_index[fam_tmp-1]-1][i+1] = cov_tmp;		
		}
	}
	for(i=0; i<n_cov; i++){
		cov_check = 0;
		for(j=0; j<n_fam; j++){
			for(k=0; k<((family+j)->n_indiv); k++){
				if(((family+j)->cov)[k][i+1] != (family->cov)[0][i+1]) cov_check=1;
			}
		}
		if(cov_check == 0){
			printf("ERROR: Phenotype file has at least one constant covariate!\n");
			exit(1);
		}
	}	
	fclose(phenofile);		
	free(indiv_index);
}
	
void readkin(char kin_filename[], int n_fam, struct FAMILY *family){
	int fam_tmp, fam_old=-1, i, j, k, index1, index2, index1old=-1, index2old=-1;
	char indiv1[MAXINDLEN], indiv2[MAXINDLEN], indiv1old[MAXINDLEN]="\0", indiv2old[MAXINDLEN]="\0";
	double coef_tmp;
	struct FAMILY *family_tmp;
	FILE *kinfile;
	if( (kinfile = fopen(kin_filename, "r"))==NULL) {
		printf("ERROR: Can't open kinship file!\n");
		exit(1);
	}
	for (i=0; i<n_fam; i++){
		family_tmp = family+i;
		family_tmp->kin = d2array(family_tmp->n_indiv, family_tmp->n_indiv);
		for(j=0; j<(family_tmp->n_indiv); j++){
			for(k=0; k<(family_tmp->n_indiv); k++){
				(family_tmp->kin)[j][k]=0;
			}
			(family_tmp->kin)[j][j]=1;
		}
	}
	while (!feof(kinfile)){
		if(fscanf(kinfile, "%d %s %s %lf", &fam_tmp, indiv1, indiv2, &coef_tmp)!=4) break;
		family_tmp = family+fam_tmp-1;
		if(fam_tmp == fam_old){
			if(strcmp(indiv1, indiv1old)==0){
				index1 = index1old;
			} else if(strcmp(indiv1, indiv2old)==0){
				index1 = index2old;
			} else {
				index1 = findInd(family_tmp->n_indiv, family_tmp->indiv, indiv1);
			}
			
			if(strcmp(indiv2, indiv2old)==0){
				index2 = index2old;
			} else if(strcmp(indiv2, indiv1old)==0){
				index2 = index1old;
			} else {
				index2 = findInd(family_tmp->n_indiv, family_tmp->indiv, indiv2);
			}
		} else{
			index1 = findInd(family_tmp->n_indiv, family_tmp->indiv, indiv1);
			index2 = findInd(family_tmp->n_indiv, family_tmp->indiv, indiv2);
		}
		if(index1!=index2){
			if((family_tmp->kin)[index1][index2]!=0&&(family_tmp->kin)[index1][index2]!=coef_tmp){
				printf("WARNING: In GRM file, the individual pair (%s and %s) is included twice, with different values! The second value, %lf, is used.\n", indiv1, indiv2, coef_tmp);
			}
		} else{
			if((family_tmp->kin)[index1][index2]!=1&&(family_tmp->kin)[index1][index2]!=coef_tmp){
				printf("WARNING: In GRM file, the diagonal element for individual %s is included twice, with different values! The second value, %lf, is used.\n", indiv1, coef_tmp);
			}
		}		
		(family_tmp->kin)[index1][index2] = coef_tmp;	
		(family_tmp->kin)[index2][index1] = coef_tmp;	
		fam_old= fam_tmp;
		index1old = index1;
		index2old = index2;
		strcpy(indiv1old, indiv1);
		strcpy(indiv2old, indiv2);
	}
}

int findInd(int n_indiv, char **indivlist, char indiv[]){
	int index = 0;
	while (index < n_indiv){
		if(strcmp(indivlist[index], indiv)==0) return (index);
		index++;
	}
	printf("ERROR: In the GRM file, individual %s is not found in phenotype data file\n", indiv);
	exit(1);
}

void eig(int n_fam, struct FAMILY *family, char eig_filename[]){
		int n_indiv, i, j;
		FILE *eigfile;
		n_indiv = (family+0)->n_indiv;
		(family+0)->eigvalue = d1array(n_indiv);
		(family+0)->eigvector = d2array(n_indiv, n_indiv);		
		if((eigfile = fopen(eig_filename, "r")) == NULL){
			printf("ERROR: Failure to open the eigen-decomposition file: %s\n", eig_filename);
			exit(1);
		}
		for(i=0; i<n_indiv; i++){
			fread(&(((family+0)->eigvalue)[i]), sizeof(double), 1, eigfile);
		}
		for(i=0; i<n_indiv; i++){
			for(j=0;j<n_indiv;j++){
				fread(&(((family+0)->eigvector)[i][j]), sizeof(double), 1, eigfile);
			}
		}
		fclose(eigfile);
}

void eigcomp_readgeno(int n_fam, int n_indiv_total, struct FAMILY *family, char eiggeno_filename[]){
	int length, n_indiv, n, count, i, j, k, n_snp=0, chr, cm, geno_allele, geno_tmp1, geno_tmp2, check_geno, *missing, n_missing, **pair_nonmissing;
	long int bp;
	double *geno, freq;
	char *line, tmp_str[15], **indiv_list, snp[MAXMARKERLEN];
	FILE *eiggenofile;
	if((eiggenofile = fopen(eiggeno_filename, "r")) == NULL){
		printf("ERROR: Failure to open GRM genotype file: %s\n", eiggeno_filename);
		exit(1);
	}		
		
		
	/* read header line */
	length = MAXINDLEN*n_indiv_total+15;
	line = (char*)malloc(length*sizeof(char));
	fgets(line, length, eiggenofile);
	n = sscanf(line,"%s %s %s %s %[^\n]", tmp_str, tmp_str, tmp_str, tmp_str, line);
	if (n<5) {
		printf("ERROR: GRM genotype file is not correctly formatted. Check the first line.\n");
		exit(1); 
	}
	indiv_list = (char**)malloc(n_indiv_total*sizeof(char*));
	for(i=0; i<n_indiv_total; i++){
		indiv_list[i] = (char*)malloc(MAXINDLEN*sizeof(char));
		n=sscanf(line, "%s %[^\n]", indiv_list[i], line);
		if(n<1) {
			printf("ERROR: In GRM genotype file, the number of individuals does not match the sample size!\n");
			exit(1);
		}
	}
	free(line);
	count=0;
	for(i=0; i<n_fam; i++){
		for(j=0; j<((family+i)->n_indiv); j++){
			if(strcmp(indiv_list[count], ((family+i)->indiv)[j])!=0){
				printf("ERROR: The list of individuals in the GRM genotype file must match exactly with the list of individuals in the phenotype file.\n");
				exit(1);
			}
			count++;
		}
	}		
	for(i=0; i<n_indiv_total; i++) free(indiv_list[i]);
	free(indiv_list);	
	
	length = MAXGENOLEN*n_indiv_total+MAXMARKERLEN;
	line = (char*)malloc(length*sizeof(char));	
	for (i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;
		(family+i)->kin = d2array(n_indiv, n_indiv);
		pair_nonmissing = i2array(n_indiv, n_indiv);
		for(j=0; j<n_indiv; j++){
			for(k=0; k<n_indiv; k++){
				((family+i)->kin)[j][k] = 0;
				pair_nonmissing[j][k] = 0;
			}
		}
		geno = d1array(n_indiv);
		missing = i1array(n_indiv);
		
		while (!feof(eiggenofile)){		
			fgets(line, length, eiggenofile);	
			n = sscanf(line,"%d %s %d %ld %[^\n]", &chr, snp, &cm, &bp, line);
			if (n<5) {	
				break; //stop when reach the last line of the genotype file
			}
			
			freq=0;
			check_geno = 0;	
			for(j=0; j<n_indiv; j++) missing[j] = 0;			
			count = 0;
			n_missing = 0;
			geno_allele = -9;
			while(geno_allele==-9 && count<n_indiv_total){
				count++;
				n = sscanf(line,"%d %d %[^\n]", &geno_tmp1, &geno_tmp2, line);
				if (n<3) {	
					printf("WARNING: In GRM genotype file, the number of individuals for the %dth SNP, %s, does not match the sample size! The SNP is ignored!\n", n_snp+1, snp);
					check_geno = 1;
					break;
				}
				if((geno_tmp1!=1&&geno_tmp1!=2&&geno_tmp1!=-9)||(geno_tmp2!=1&&geno_tmp2!=2&&geno_tmp2!=-9)){
					printf("WARNING: In GRM genotype file, at least one allele at SNP %s is not labeled by 1, 2 or -9 (missing alleles should be coded as -9)! The SNP is ignored!\n", snp);
					check_geno = 1;
					break;
				}
				geno[count-1] = 0;
				if(geno_tmp1!=-9){
					geno_allele = geno_tmp1;
					if(geno_tmp2 == -9){
						missing[count-1]++;
						geno[count-1] = 0;
						n_missing++;
					}else if(geno_tmp2 != geno_allele){
						geno[count-1] = 1;
					}
				} else{
					n_missing++;
					missing[count-1]++;
					geno[count-1] = 0;
					if(geno_tmp2 == -9){
						missing[count-1]++;
					}else{
						geno_allele = geno_tmp2;
					}				
				}
				freq += geno[count-1];
			}			
			if(check_geno == 1) continue;
			if(geno_allele==-9){
				printf("WARNING: In GRM genotype file, all genotypes at the %dth SNP, %s, are missing! The SNP is ignored!\n", n_snp+1, snp);
				continue;
			}  else {
				while (n==3){
					n=sscanf(line,"%d %d %[^\n]", &geno_tmp1, &geno_tmp2, line);
					if (n<2) break;
					(count)++;	
					geno[count-1]=0;
					missing[count-1] = 0;
					if(count<=n_indiv_total){
						if((geno_tmp1!=1&&geno_tmp1!=2&&geno_tmp1!=-9)||(geno_tmp2!=1&&geno_tmp2!=2&&geno_tmp2!=-9)){
							printf("WARNING: In GRM genotype file, at least one allele at SNP %s is not labeled by 1, 2 or -9 (missing alleles should be coded as -9)! The SNP is ignored!\n", snp);
							check_geno = 1;
						}
						if(geno_tmp1==-9){
							missing[count-1]++;
						} else if(geno_tmp1!=geno_allele){
							geno[count-1]++;
						}
						if(geno_tmp2==-9){
							missing[count-1]++;
						} else if(geno_tmp2!=geno_allele){
							geno[count-1]++;
						}	
						if(geno_tmp1==-9||geno_tmp2==-9) {
							geno[count-1]=0;
							n_missing++;
						}
						freq += geno[count-1];
					} else{
						printf("WARNING: In GRM genotype file, number of individuals for SNP %s does not match the sample size! The SNP is ignored!\n", snp);
						break;
					}
				}
			}	
			if(count != n_indiv_total){
				printf("WARNING: In genotype file, number of individuals for SNP %s does not match the sample size! The SNP is ignored!\n", snp);
				continue;
			}
			if(check_geno == 1){
				continue;
			}
			freq /= 2*n_indiv - n_missing;
			for(j=0; j<n_indiv; j++) geno[j] -= 2*freq;
			for(j=0; j<n_indiv; j++){
				for(k=j; k<n_indiv; k++){
					if(missing[j]==0&&missing[k]==0){
						((family+i)->kin)[j][k] += geno[j]*geno[k]/(2*freq*(1-freq));
						pair_nonmissing[j][k]++;
					}					
				}
			}
		}
		free(geno);
		for(j=0; j<n_indiv; j++){
			for(k=j; k<n_indiv; k++){
				if(pair_nonmissing[j][k]<2){
					if(j!=k){
						if(pair_nonmissing[j][k]==0){
							printf("ERROR: In GRM genotype file, no SNP has non-missing genotypes at both individuals %d and %d. Genetic relationship cannot be estimated between these individuals. Consider removing them from the analysis.\n", j, k);
							exit(1);
						} else{
							printf("Warning: In GRM genotype file, only %d SNP(s) has non-missing genotypes at both individuals %d and %d. The estimated genetic relationship may not be reliable between these individuals. \n", pair_nonmissing[j][k], j, k);
						}
					} else{
						if(pair_nonmissing[j][k]==0){
							printf("WARNING: In GRM genotype file, no SNP has non-missing genotype at individual %d. In the estimated GRM, 1 is used for the corresponding diagonal element.\n", j);
						} else{
							printf("WARNING: In GRM genotype file, only %d SNP has non-missing genotype at individual %d. In the estimated GRM, the corresponding diagonal element may not be reliable.\n", pair_nonmissing[j][k], j);
						}						
					}
				}
				if(pair_nonmissing[j][k]>0){
					((family+i)->kin)[j][k] /= pair_nonmissing[j][k];
				} else if (j==k){
					((family+i)->kin)[j][k] = 1;
				}
			}
			for(k=0; k<j; k++){
				((family+i)->kin)[j][k] = ((family+i)->kin)[k][j];
			}			
		}
		free(missing);
		free_i2array(pair_nonmissing, n_indiv, n_indiv);
	}
	n_indiv = (family+0)->n_indiv;
	(family+0)->eigvalue = d1array(n_indiv);
	(family+0)->eigvector = d2array(n_indiv, n_indiv);	
	fclose(eiggenofile);
	free(line);
}

void eigcomp_writeGRM(int n_fam, struct FAMILY *family, char *outgrm_filename){
	int i, n_indiv, j, k;
	struct FAMILY *family_tmp;
	FILE *outgrmfile;
	outgrmfile = fopen(outgrm_filename, "w");
	for(i=0; i<n_fam; i++){
		family_tmp = family + i;
		n_indiv = family_tmp->n_indiv;
		for(j=0; j<n_indiv; j++){
			for(k=j; k<n_indiv; k++){
				fprintf(outgrmfile, "1 %s %s %.6lf\n", (family_tmp->indiv)[j], (family_tmp->indiv)[k], (family_tmp->kin)[j][k]);
			}
		}
	}
	fclose(outgrmfile);
}

void eigcomp_kin(int n_fam, struct FAMILY *family, char outeig_filename[]){
	int i, j, k, n_indiv;
	FILE *outeigfile;
	outeigfile = fopen(outeig_filename, "w");
	for (i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;
		(family+i)->eigvalue = d1array(n_indiv);
		(family+i)->eigvector = d2array(n_indiv, n_indiv);
		MapRowXd phi_map (((family+i)->kin)[0], n_indiv, n_indiv);
		SelfAdjointEigenSolver<MatrixXd> es;
		es.compute(phi_map);
		for(j=0; j<n_indiv; j++) {
			((family+i)->eigvalue)[j] = es.eigenvalues()(j);
			for(k=0; k<n_indiv; k++){
				((family+i)->eigvector)[j][k] = es.eigenvectors()(j, k);
			}
			fwrite(&(((family+i)->eigvalue)[j]), sizeof(double), 1, outeigfile);
		}
	}
	for(i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;
		for(j=0; j<n_indiv; j++) {
			for(k=0; k<n_indiv; k++){
				fwrite(&(((family+i)->eigvector)[j][k]), sizeof(double), 1, outeigfile);
			}
		}
	}				
	fclose(outeigfile);
}

void leigcomp_kin(int n_fam, struct FAMILY *family, char outeig_filename[]){
	int i, n_indiv, j, k;
	int il=1, vrsize, evf, info, lwork, liwork, iwkopt, *iwork, *isuppz;
	double wkopt, *work, vl=0, vu=0, *w, *z, abstol=-1.0;
	struct FAMILY *family_tmp;
	for (i=0; i<n_fam; i++){
		family_tmp = family+i;
		n_indiv = (family+i)->n_indiv;
		family_tmp->eigvalue = d1array(n_indiv);
		family_tmp->eigvector = d2array(n_indiv, n_indiv);

		w = (double*)malloc( n_indiv*sizeof(double) );
		z = d1array(n_indiv*n_indiv);				
		isuppz = (int*)malloc( 2*n_indiv*sizeof(int) );			
		if (savemem==1){
			lwork = 1 + 26*n_indiv;
			liwork = 1 + 10*n_indiv;
		} else {
			lwork=-1;
			liwork=-1;			
			dsyevr_( "V", "A", "U", &n_indiv, family_tmp->kin[0], &n_indiv, &vl, &vu, &il, &n_indiv, &abstol, &evf, w, z, &n_indiv, isuppz, &wkopt, &lwork, &iwkopt, &liwork, &info );
        		lwork = (int)wkopt;
			liwork = (int)iwkopt;
		}
        	work = (double*)malloc( lwork*sizeof(double) );
		iwork = (int*)malloc( liwork*sizeof(int) );
		dsyevr_( "V", "A", "U", &n_indiv, family_tmp->kin[0], &n_indiv, &vl, &vu, &il, &n_indiv, &abstol, &evf, w, z, &n_indiv, isuppz, work, &lwork, iwork, &liwork, &info );
		if( info > 0 ) {
                printf( "The algorithm failed to compute eigenvalues for the kinship matrix of family # %d.\n", i);
                return;
        	}
		for (j=0; j<n_indiv; j++){
			(family_tmp->eigvalue)[j] = w[j];
			for(k=0; k<n_indiv; k++){
				(family_tmp->eigvector)[j][k] = (z)[j+k*n_indiv];
			}
		}
		free(w);
		free(z);			
		free(isuppz);						
		free( work );
		free(iwork);
	}
        return;
}

void Logistic (int n_fam, int n_cov, int n_indiv_total, struct FAMILY *family, struct PARAM *param){
	int i, n_indiv, count=0, j, k, itermax = 100, itercount=0;
	double diff, tol1=1e-6, tol2=1e-6;
	n_cov++;
	MatrixXd y(n_indiv_total, 1);
	MatrixXd x(n_indiv_total, n_cov);
	MatrixXd z(n_indiv_total, 1);
	MatrixXd w(n_indiv_total, 1);
	MatrixXd eta(n_indiv_total, 1);
	MatrixXd eta2(n_indiv_total, 1);
	MatrixXd mu(n_indiv_total, 1);
	MatrixXd mu2(n_indiv_total, 1);
	MatrixXd xtw(n_cov, n_indiv_total);
	MatrixXd beta2(n_cov, 1);
	for(i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;
		for(j=0; j<n_indiv; j++){
			y(count, 0) = ((family+i)->pheno)[j];
			(param->beta_initial)[0] += y(count, 0);
			for(k=0; k<n_cov; k++){
				x(count, k) = ((family+i)->cov)[j][k];
			}
			count++;
		}
	}
	(param->beta_initial)[0] /= n_indiv_total+1;
	(param->beta_initial)[0] = log((param->beta_initial)[0]/(1-(param->beta_initial)[0]));
	MapRowXd beta_map ( param->beta_initial, n_cov, 1); 
	eta = x * beta_map;
	mu = (1 - 1/(eta.array().exp()+1)).matrix();
	while (itercount<itermax){
		itercount++;
		w = (mu.array()*(1-mu.array())).matrix();
		xtw = x.transpose() * w.asDiagonal();
		z = eta + ((1/w.array())).matrix().asDiagonal() * (y-mu);
		beta2 = (xtw * x).inverse() * (xtw * z);
		eta2 = x * beta2;
		if((beta_map-beta2).squaredNorm()<tol1) break;
		mu2 = (1 - 1/(eta2.array().exp()+1)).matrix();
		if((mu-mu2).squaredNorm()<tol2) break;
		beta_map = beta2;
		eta = eta2;
		mu = mu2;		
	}
}

/**********************************************************************************************************/
/**** Iterative parameter estimation for beta and xi, under H0 (sigmaT2 = 1 for CARAT) *********************/
/**** Returns 0 if convergence attained for Newton-Rhapson search of beta, 1 if failed. *******************/
/**********************************************************************************************************/	
int estimate(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param, char param_filename[]){
	int beta_converge, i;
	FILE *paramfile;
	optimize_xi (n_fam, n_cov, family, param, &beta_converge);
	paramfile = fopen(param_filename, "w");
	fprintf(paramfile, "Parameter\tNull_Estimate\n");
	fprintf(paramfile, "Variance_Parameter_xi\t%.4e\n", *(param->xi));
	fprintf(paramfile, "Intercept\t%.4e\n", (param->beta)[0]);
	for(i=1; i<n_cov+1; i++) fprintf(paramfile, "Covariate_%d\t%.4e\n", i, (param->beta)[i]);
	return beta_converge;
}	
	
/**********************************************************************************************************/
/**** Estimate beta given xi, using Newton-Rhapson *******************************************************/
/**** Returns 0 if convergence attained, 1 if failed. *****************************************************/
/**** tol1 = precision in mu, tol2 = precision in beta, max_iter = maximum interactions allowed ***********/
/**********************************************************************************************************/
int estimate_Beta(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param){
	int i, n_indiv, j, n_iter=0, max_iter=50;
	double sigmaT, xi, mudiff=1, betadiff=1, tol1=1e-6, tol2=1e-6;
	xi = *(param->xi);
	n_cov++;
	
	for (i=0; i<n_cov; i++) (param->beta)[i] = (param->beta_initial)[i];
	
	MatrixXd beta (n_cov, 1);
	MatrixXd XtWX (n_cov, n_cov);
	MatrixXd XtM_SigmaInv_MInv_y(n_cov, 1);
	MatrixXd beta_new (n_cov, 1);
	
	while (mudiff>tol1 && n_iter<max_iter){
		n_iter++;
		for (i=0; i<n_cov; i++) beta(i,0) = (param->beta)[i];
		XtWX = MatrixXd::Zero(n_cov, n_cov);
		XtM_SigmaInv_MInv_y = MatrixXd::Zero(n_cov, 1);
		for (i=0; i<n_fam; i++){
			n_indiv = (family+i)->n_indiv;
			MapRowXd x_map ( ((family+i) -> cov)[0], n_indiv, n_cov); 
			MapRowXd y_map ( (family+i) -> pheno, n_indiv, 1); 		
			MapColXd Ut_map ( ((family+i) -> eigvector)[0], n_indiv, n_indiv); 	
			MapRowXd D_inv_map ( (family+i)->D_inv, n_indiv, 1);
			MatrixXd mu = (1 - 1/((x_map*beta).array().exp().array() + 1)).matrix(); 		
			MatrixXd M = (mu.array()*(1-mu.array())).sqrt().matrix(); 		
			MatrixXd M_inv(n_indiv, 1);
			for(j=0; j<n_indiv; j++){
				if(M(j,0)>1e-12 && M(j,0)<1-1e-12) {
					M_inv(j,0) = 1/M(j,0);
				} else{
					M_inv(j,0) = 1e12;
					M(j,0) = 1e-12;
				}
			}
			//MatrixXd xtMU = (x_map.transpose() * M.asDiagonal()) * U_map;
			MatrixXd UtMx = Ut_map *(M.asDiagonal() * x_map); 	
			XtWX = XtWX + UtMx.transpose() * D_inv_map.asDiagonal() * UtMx; 			
			XtM_SigmaInv_MInv_y = XtM_SigmaInv_MInv_y + UtMx.transpose() * ( D_inv_map.asDiagonal() * (Ut_map * (M_inv.asDiagonal() * (y_map-mu))) ); 	
		}
		beta_new = beta + XtWX.inverse() * XtM_SigmaInv_MInv_y;
		betadiff = (beta-beta_new).array().abs().sum();
		mudiff = 0;
		for (i=0; i<n_fam; i++){
			n_indiv = (family+i)->n_indiv;
			MapRowXd x_map ( ((family+i) -> cov)[0], n_indiv, n_cov);
			ArrayXd mu = 1 - 1/((x_map*beta).array().exp().array() + 1);
			ArrayXd mu_new = 1 - 1/((x_map*beta_new).array().exp().array() + 1);
			mudiff += (mu - mu_new).abs().sum();
		}
		for (i=0; i<n_cov; i++) (param->beta)[i] = beta_new(i,0);
	}
	for (i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;	
		n_indiv = (family+i)->n_indiv;
		MapRowXd x_map ( ((family+i) -> cov)[0], n_indiv, n_cov);
		MapRowXd y_map ( (family+i) -> pheno, n_indiv, 1); 		
		MapColXd Ut_map ( ((family+i) -> eigvector)[0], n_indiv, n_indiv); 	
		MatrixXd mu = 1 - 1/((x_map*beta_new).array().exp().array() + 1);
		MatrixXd M = (mu.array()*(1-mu.array())).sqrt().matrix(); 		
		VectorXd M_inv(n_indiv);
		for(j=0; j<n_indiv; j++){
			if(M(j,0)>1e-12 && M(j,0)<1-1e-12) {
				M_inv(j) = 1/M(j,0);
			} else{
				M_inv(j) = 1e12;
				M(j,0) = 1e-12;
//				printf("M blows\n");
			}
		}		
		MatrixXd Z = Ut_map * M_inv.asDiagonal() * (y_map - mu);
		for (j=0; j<n_indiv; j++) ((family+i)->Z)[j] = Z(j,0);
	}
	
	if (mudiff<tol1) {
		return 0;
	}
	return 1;
}

/******************************************************************************************************/
/**** This function is needed only for CARATd *********************************************************/
/**** Estimate sigmaT2 for known beta and xi **********************************************************/
/**** Relies on familywise Z and D_inv, as output by estimate_Beta ************************************/
/******************************************************************************************************/				
void estimate_sigmaT2(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param){
	int i, n_indiv, n_indiv_total=0, j;
	n_cov++;
	*(param->sigmaT2) = 0;
	for (i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;
		n_indiv_total += n_indiv;
		MatrixXd Z(n_indiv, 1);
		for (j=0; j<n_indiv; j++) Z(j,0) = ((family+i)->Z)[j];
		MapRowXd D_inv_map ( (family+i)->D_inv, n_indiv, 1); 	
		*(param->sigmaT2) += (Z.transpose() * D_inv_map.asDiagonal() * Z)(0,0);
	}	
	*(param->sigmaT2) /= n_indiv_total - n_cov;
	printf("sigma=%lf\n", *(param->sigmaT2));
}	

/**********************************************************************************************************/
/**** For given xi, beta (and possibly sigmaT2), **********************************************************/
/**** returns a xi score, as defined by fabs(LHS-RHS) in the estimating equation for xi *******************/
/**********************************************************************************************************/
double xi_score(double xi, void *info){
	struct FAMILY *family, *family_tmp;
	double sigmaT2, LHS=0, RHS=0;
	int n_fam, i, n_indiv, j;
	family = ((struct INFO*)info)->family;
	sigmaT2 = *((((struct INFO*)info)->param)->sigmaT2);
	n_fam = ((struct INFO*)info)->n_fam;
	for(i=0; i<n_fam; i++){
		family_tmp = family+i;
		n_indiv = family_tmp -> n_indiv;
		for (j=0; j<n_indiv; j++){
			LHS += pow((family_tmp->Z)[j] * (family_tmp->D_inv)[j],2) * ((family_tmp->eigvalue)[j]-1);
			RHS += ((family_tmp->eigvalue)[j]-1) * (family_tmp->D_inv)[j];
		}
	}
	LHS /= sigmaT2;
	return fabs(LHS-RHS);
}

/**********************************************************************************************************/
/**** For given xi and the corresponding optimal beta (and possibly sigmaT2)*******************************/
/**** returns the xi score for this xi, as given in xi_score **********************************************/
/**********************************************************************************************************/
double xi_score_optim(double xi, void *info){
	struct FAMILY *family, *family_tmp;
	struct PARAM *param;
	int n_fam, n_cov, i, n_indiv, j;
	family = ((struct INFO*)info)->family;
	n_fam = ((struct INFO*)info)->n_fam;
	n_cov = ((struct INFO*)info)->n_cov;
	param = ((struct INFO*)info)->param;
	*(param->xi) = xi;
	for(i=0; i<n_fam; i++){
		family_tmp = family+i;
		for (j=0; j<family_tmp->n_indiv; j++){
			(family_tmp -> D_inv)[j] = xi * (family_tmp->eigvalue)[j] + 1 - xi;
			(family_tmp -> D_inv)[j] = 1/(family_tmp -> D_inv)[j];
		}
	}	
	*(((struct INFO*)info)->beta_converge) = estimate_Beta(n_fam, n_cov, family, param);
//	estimate_sigmaT2(n_fam, n_cov, family, param);	
    *(param->sigmaT2) = 1;
	return xi_score(xi, info);
}

/**********************************************************************************************************/
/**** Optimize xi over [0,1] to obtain minimized xi score as given in xi_score_optim *********************/
/**********************************************************************************************************/
void optimize_xi (int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param, int *beta_converge){
	struct INFO *info;
	double xi, tol=1e-6;
	info = (struct INFO *)malloc(sizeof(struct INFO));
	info->family = family;
	info->param = param;
	info->n_fam = n_fam;
	info->n_cov = n_cov;
	info->beta_converge = beta_converge;
	xi = Brent_fmin(0, 1, xi_score_optim, info, tol);
	*(param->xi) = xi;
	*(beta_converge) = *(info->beta_converge);
}
	
void PrepareTest(int n_fam, int n_cov, struct FAMILY *family, struct PARAM *param, double *varT){
	int i, n_indiv, j, k;
	double xi;
	n_cov++;
	*varT = 0;
	xi = *(param->xi);
	MatrixXd beta (n_cov, 1);
	for(k=0; k<n_cov; k++) beta(k,0) = (param->beta)[k];
	for (i=0; i<n_fam; i++){
		n_indiv = (family+i)->n_indiv;
		for (j=0; j<n_indiv; j++){
			((family+i)->D_inv)[j] = 1 / (xi * (family+i)->eigvalue[j] + 1 - xi);			
		}
		MapRowXd x_map ( ((family+i) -> cov)[0], n_indiv, n_cov); 
		MapRowXd y_map ( (family+i) -> pheno, n_indiv, 1); 		
		MapRowXd U_map ( ((family+i) -> eigvector)[0], n_indiv, n_indiv); 	
		MatrixXd mu = (1 - 1/((x_map*beta).array().exp().array() + 1)).matrix(); 		
		MatrixXd M = (mu.array()*(1-mu.array())).sqrt().matrix(); 
		MatrixXd M_inv(n_indiv, 1);
		MapRowXd D_inv_map ( (family+i) -> D_inv, n_indiv, 1); 
		MapRowXd L_map ((family+i) -> eigvalue, n_indiv, 1);
		for(j=0; j<n_indiv; j++){
			((family+i)->M)[j] = M(j,0);
			if(M(j,0)>1e-12 && M(j,0)<1-1e-12) {
				M_inv(j,0) = 1/M(j,0);
			} else{
				M_inv(j,0) = 1e12;
			}
		}
		MatrixXd y4 = (y_map - mu).transpose() * M_inv.asDiagonal() * U_map * D_inv_map.asDiagonal() * U_map.transpose() * M.asDiagonal(); 
		MatrixXd tmp = y4 * U_map;
		*varT += (tmp * L_map.asDiagonal() * tmp.transpose())(0,0);
		for (j=0; j<n_indiv; j++){
			((family+i)->y4)[j] = y4(0,j);
		}
	}
}

int Test(int n_fam, int n_cov, int n_indiv_total, char geno_filename[], char p_filename[], 
	struct FAMILY *family, struct PARAM *param, double *varT){
	
	int i, length, chr, cm, tmp, n=0, count=0, n_snp=0, n_indiv, j, k, geno_allele, geno_tmp1, geno_tmp2, check_geno, n_missing, *missing;
	long int bp;
	double *G, T=0, p,sigma2, pvalue;
	char tmp_str[15], **indiv_list, *line, indiv_tmp[MAXINDLEN], marker[MAXMARKERLEN];
	struct FAMILY *family_tmp;
	n_cov++;
	FILE *genofile, *pfile;
	if( (genofile = fopen(geno_filename, "r"))==NULL) {
		printf("ERROR: Can't open genotype file!\n");
		exit(1);
	}
	
	/* read header line */
	length = MAXINDLEN*n_indiv_total+15;
	line = (char*)malloc(length*sizeof(char));
	fgets(line, length, genofile);
	n = sscanf(line,"%s %s %s %s %[^\n]", tmp_str, tmp_str, tmp_str, tmp_str, line);
	if (n<5) {
		printf("ERROR: Genotype file is not correctly formatted. Check the first line.\n");
		exit(1); 
	}
	indiv_list = (char**)malloc(n_indiv_total*sizeof(char*));
	for(i=0; i<n_indiv_total; i++){
		indiv_list[i] = (char*)malloc(MAXINDLEN*sizeof(char));
		n=sscanf(line, "%s %[^\n]", indiv_list[i], line);
		if(n<1) {
			printf("ERROR: In genotype file, the number of individuals does not match the sample size!\n");
			exit(1);
		}
	}
	free(line);
	count=0;
	for(i=0; i<n_fam; i++){
		family_tmp = family+i;
		for(j=0; j<(family_tmp->n_indiv); j++){
			if(strcmp(indiv_list[count], (family_tmp->indiv)[j])!=0){
				printf("ERROR: The list of individual in the genotype file must match exactly with the list of individuals in the phenotype file.\n");
				exit(1);
			}
			count++;
		}
	}

	pfile = fopen(p_filename, "w");
	fprintf(pfile, "Chr SNP BP p-value\n");
	length = MAXGENOLEN*n_indiv_total+MAXMARKERLEN;
	line = (char*)malloc(length*sizeof(char));
	G = d1array(n_indiv_total);	
	missing = i1array(n_indiv_total);
	while (!feof(genofile)){
		fgets(line, length, genofile);	
		n = sscanf(line,"%d %s %d %ld %[^\n]", &chr, marker, &cm, &bp, line);
		if (n<5) {	
			break; //stop when reach the last line of the genotype file
		}
		n_missing = 0;
		p=0;
		check_geno = 0;
		for(i=0; i<n_indiv_total; i++) missing[i] = 0;
		count = 0;
		geno_allele = -9;
		while(geno_allele==-9 && count<n_indiv_total){
			count++;
			n = sscanf(line,"%d %d %[^\n]", &geno_tmp1, &geno_tmp2, line);
			if (n<3) {	
				printf("WARNING: In genotype file, the number of individuals for the %dth SNP, %s, does not match the sample size! The SNP is ignored!\n", n_snp+1, marker);
				check_geno = 1;
				break;
			}
			if((geno_tmp1!=1&&geno_tmp1!=2&&geno_tmp1!=-9)||(geno_tmp2!=1&&geno_tmp2!=2&&geno_tmp2!=-9)){
					printf("WARNING: In genotype file, at least one allele at SNP %s is not labeled by 1, 2 or -9 (missing alleles should be coded as -9)! The SNP is ignored!\n", marker);
					check_geno = 1;
					break;
			}
			if(geno_tmp1!=-9){
				geno_allele = geno_tmp1;
				G[count-1] = 0;
				if(geno_tmp2 == -9){
					n_missing++;
					missing[count-1]++;
				}else if(geno_tmp2 != geno_allele){
					G[count-1]++;
				}
			} else{
				n_missing++;
				missing[count-1]++;
				if(geno_tmp2 == -9){
					n_missing++;
					missing[count-1]++;
				}else{
					geno_allele = geno_tmp2;
					G[count-1] = 0;
				}				
			}
			p += G[count-1];
		}
		if(check_geno == 1) continue;
		if(geno_allele==-9){
			printf("WARNING: In genotype file, all genotypes at the %dth SNP, %s, are missing! The SNP is ignored!\n", n_snp+1, marker);
			continue;
		} else {
			while (n==3){
				n=sscanf(line,"%d %d %[^\n]", &geno_tmp1, &geno_tmp2, line);
				if (n<2) break;
				(count)++;	
				G[count-1]=0;
				missing[count-1] = 0;
				if(count<=n_indiv_total){
					if((geno_tmp1!=1&&geno_tmp1!=2&&geno_tmp1!=-9)||(geno_tmp2!=1&&geno_tmp2!=2&&geno_tmp2!=-9)){
						printf("WARNING: In genotype file, at least one allele at SNP %s is not labeled by 1, 2 or -9 (missing alleles should be coded as -9)! The SNP is ignored!\n", marker);
						check_geno = 1;
					}
					if(geno_tmp1==-9){
						missing[count-1]++;
						n_missing++;
					} else if(geno_tmp1!=geno_allele){
						G[count-1]++;
					}
					if(geno_tmp2==-9){
						missing[count-1]++;
						n_missing++;
					} else if(geno_tmp2!=geno_allele){
						G[count-1]++;
					}					
					p += G[count-1];
				} else{
					printf("WARNING: In genotype file, number of individuals for SNP %s does not match the sample size! The SNP is ignored!\n", marker);
					break;
				}
			}
		}	
		if(count != n_indiv_total){
			printf("WARNING: In genotype file, number of individuals for SNP %s does not match the sample size! The SNP is ignored!\n", marker);
			continue;
		}
		if(check_geno == 1){
			continue;
		}
		p /= 2*n_indiv_total - n_missing;
		sigma2 = 0;
		for(i=0; i<n_indiv_total; i++){
			G[i] += p * missing[i];
			sigma2 += pow(G[i]-2*p, 2);
		}
		sigma2 /= n_indiv_total -1;
		count =0;
		T=0;
		for (i=0; i<n_fam; i++){
			family_tmp = family+i;
			n_indiv = family_tmp->n_indiv;
			for(j=0; j<n_indiv; j++) T += (family_tmp->y4)[j]*G[count+j];
			count += n_indiv;
		}
		if(n_missing==0){
			T = pow(T,2)/((*varT)*2*p*(1-p));
		} else{
			T = pow(T,2) / (sigma2 * (*varT));
		}
		pvalue = gsl_cdf_ugaussian_Q(sqrt(T))*2;
		n_snp++;
		fprintf(pfile, "%d %s %ld %.4e \n", chr, marker, bp, pvalue);
	}
	printf("Association tests completed for %d SNPs. Results stored in %s\n", n_snp, p_filename);
	fclose(genofile);
	fclose(pfile);
	free(line);
	free(G);
	free(missing);
	for(i=0; i<n_indiv_total; i++) free(indiv_list[i]);
	free(indiv_list);
}

/*	
int Test_bckup(int n_fam, int n_cov, int n_indiv_total, char geno_filename[], char p_filename[], 
	struct FAMILY *family, struct PARAM *param, double *varT){
	
	int i, length, chr, cm, tmp, n=0, count=0, n_snp=0, n_indiv, j, k;
	long int bp;
	double *G, geno_tmp, T=0, p, pvalue;
	char tmp_str[15], **indiv_list, *line, indiv_tmp[MAXINDLEN], marker[MAXMARKERLEN];
	struct FAMILY *family_tmp;
	n_cov++;
	FILE *genofile, *pfile;
	if( (genofile = fopen(geno_filename, "r"))==NULL) {
		printf("Can't open genotype file!\n");
		exit(1);
	}
	
	// read header line 
	length = MAXINDLEN*n_indiv_total+15;
	line = (char*)malloc(length*sizeof(char));
	fgets(line, length, genofile);
	n = sscanf(line,"%s %s %s %s %[^\n]", tmp_str, tmp_str, tmp_str, tmp_str, line);
	if (n<5) {
		printf("ERROR: Genotype file is not correctly formatted. Check the first line.\n");
		exit(1); //stop when reach the last line of the genotype file
	}
	indiv_list = (char**)malloc(n_indiv_total*sizeof(char*));
	for(i=0; i<n_indiv_total; i++){
		indiv_list[i] = (char*)malloc(MAXINDLEN*sizeof(char));
		n=sscanf(line, "%s %[^\n]", indiv_list[i], line);
		if(n<1) {
			printf("ERROR: In genotype file, the number of individuals does not match the sample size!\n");
			exit(1);
		}
	}
	free(line);

	pfile = fopen(p_filename, "w");
	length = MAXGENOLEN*n_indiv_total+MAXMARKERLEN;
	line = (char*)malloc(length*sizeof(char));
	G = d1array(n_indiv_total);	
	while (!feof(genofile)){
		fgets(line, length, genofile);	
		n = sscanf(line,"%d %s %d %ld %[^\n]", &chr, marker, &cm, &bp, line);
		if (n<5) {	
			break; //stop when reach the last line of the genotype file
		}
		n = sscanf(line,"%lf %[^\n]", &geno_tmp, line);
		if (n<2) {	
			printf("WARNING: In genotype file, the number of individuals for the %dth marker does not match the sample size!\n", n_snp+1);
			break;
		} else {
			count = 1;
			p=0;
			G[0] = geno_tmp;
			p += geno_tmp;
			while (n==2){
				n=sscanf(line,"%lf %[^\n]", &geno_tmp, line);
				if (n<1) break;
				(count)++;	
				if(count<=n_indiv_total){
					G[count-1] = geno_tmp;
					p += geno_tmp;
				} else{
					printf("WARNING: In genotype file, number of individuals for the %dth marker does not match the sample size!\n", n_snp+1);
					exit(1);
					break;
				}
			}
		}	
		if(count != n_indiv_total){
			printf("WARNING: In genotype file, number of individuals for the %dth marker does not match the sample size!\n", n_snp+1);
			exit(1);
			break;
		}
		p /= 2*n_indiv_total;
		count =0;
		T=0;
		for (i=0; i<n_fam; i++){
			family_tmp = family+i;
			n_indiv = family_tmp->n_indiv;
			for(j=0; j<n_indiv; j++) T += (family_tmp->y4)[j]*G[count+j];
			count += n_indiv;
		}
		T = pow(T,2)/((*varT)*2*p*(1-p));
		pvalue = gsl_cdf_chisq_Q (T, 1);
		n_snp++;
		fprintf(pfile, "%d %s %ld %lf \n", chr, marker, bp, pvalue);
	}
	printf("Association tests completed for %d markers. Results found in %s\n", n_snp, p_filename);
	fclose(genofile);
	fclose(pfile);
	free(line);
	free(G);
	for(i=0; i<n_indiv_total; i++) free(indiv_list[i]);
	free(indiv_list);
}
*/